// FourierPDEView.cpp : implementation of the CFourierPDEView class
//

#include "stdafx.h"
#include "FourierPDE.h"

#include "FourierPDEDoc.h"
#include "FourierPDEView.h"
#include "ParameterDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Thread for simulation: otherwise it freezes

UINT RunThread(LPVOID voidPView)
{
	CFourierPDEView * pView = (CFourierPDEView *)voidPView;
	PDEsim &sim = pView->sim;
	CClientDC dc(pView);

	int i,j;
	
	while(!pView->stopThread)
	{
		
		double min, max;

		if (pView->sim.GetMin() < pView->sim.GetMax())
		{
			max = pView->sim.GetMax();
			min = pView->sim.GetMin();
		}
		else
		{
			max = -1e99, min = 1e99;

			for (i=0; i<sim.GetDim(0); i++)
			{
				for (j=0; j<sim.GetDim(1); j++)
				{
					if (sim.GetPsi(i,j)>max) max=sim.GetPsi(i,j);
					if (sim.GetPsi(i,j)<min) min=sim.GetPsi(i,j);
				}
			}
		}
	
		for (i=0; i<sim.GetDim(0); i++)
		{
			for (j=0; j<sim.GetDim(1); j++)
			{
				int color = 128;
				if (max!=min)
					color = (int)(256*((sim.GetPsi(i,j)-min)/(max-min)));
				if (color>255) color=255;
				if (color<0) color=0;
				pView->bufferDC->SetPixel(i,j,RGB(color,color,color));
			}
		}

		dc.BitBlt(0,0,sim.GetDim(0),sim.GetDim(1),pView->bufferDC,0,0,SRCCOPY);

		sim.Sweep();
	}
	pView->stopThread=false;	
	return 1;
}

/////////////////////////////////////////////////////////////////////////////
// CFourierPDEView

IMPLEMENT_DYNCREATE(CFourierPDEView, CView)

BEGIN_MESSAGE_MAP(CFourierPDEView, CView)
	//{{AFX_MSG_MAP(CFourierPDEView)
	ON_COMMAND(ID_PARAMETER_SET, OnParameterSet)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFourierPDEView construction/destruction

CFourierPDEView::CFourierPDEView()
{
	stopThread = false;
	// TODO: add construction code here

}

CFourierPDEView::~CFourierPDEView()
{
	stopThread = true;
	while(stopThread);
}

BOOL CFourierPDEView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CFourierPDEView drawing

void CFourierPDEView::OnDraw(CDC* pDC)
{
	CFourierPDEDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC->BitBlt(0,0,sim.GetDim(0),sim.GetDim(1),bufferDC,0,0,SRCCOPY);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CFourierPDEView printing

BOOL CFourierPDEView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CFourierPDEView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CFourierPDEView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CFourierPDEView diagnostics

#ifdef _DEBUG
void CFourierPDEView::AssertValid() const
{
	CView::AssertValid();
}

void CFourierPDEView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CFourierPDEDoc* CFourierPDEView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFourierPDEDoc)));
	return (CFourierPDEDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CFourierPDEView message handlers

void CFourierPDEView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();

	CClientDC dc(this);
	
	bufferDC = new CDC();

	buffer = new CBitmap();

	bufferDC->CreateCompatibleDC(&dc);

	buffer->CreateCompatibleBitmap(&dc,sim.GetDim(0),sim.GetDim(1));

	oldBuffer=bufferDC->SelectObject(buffer);
	
	thread = AfxBeginThread(RunThread,this);


	// TODO: Add your specialized code here and/or call the base class
	
}


void CFourierPDEView::OnParameterSet() 
{
	ParameterDialog dialog;

	dialog.m_dislocation_separation = sim.GetDislocationSeparation();
	dialog.m_dt = sim.GetDt();
	dialog.m_kx = sim.GetKx();
	dialog.m_ky = sim.GetKy();
	dialog.m_noise = sim.GetNoise();
	dialog.m_nu = sim.GetNu();
	dialog.m_seed = sim.GetSeed();
	dialog.m_min = sim.GetMin();
	dialog.m_max = sim.GetMax();
	// Swift-Hohenberg
	dialog.m_epsilon = sim.GetEpsilon();
	dialog.m_k = sim.GetK();

	// Generalized Swift-Hohenberg
	dialog.m_g_m = sim.GetG_m();
	dialog.m_c2 = sim.GetC2();

	if(dialog.DoModal()==IDOK)
	{
		stopThread = true;
		while (stopThread);
		// Wait for thread to stop

		if (dialog.m_nu != sim.GetNu())
		{
			CClientDC dc(this);
			dc.PatBlt(0,0,sim.GetDim(0),sim.GetDim(1),WHITENESS);
			sim.SetNu(dialog.m_nu);
			bufferDC->SelectObject(oldBuffer);
			delete bufferDC;
			delete buffer;
			bufferDC = new CDC;
			bufferDC->CreateCompatibleDC(&dc);
			buffer = new CBitmap();
			buffer->CreateCompatibleBitmap(&dc,sim.GetDim(0),sim.GetDim(1));
			oldBuffer=bufferDC->SelectObject(buffer);
			bufferDC->PatBlt(0,0,sim.GetDim(0),sim.GetDim(1),WHITENESS);
			OnDraw(&dc);
		}
		sim.SetDislocationSeparation(dialog.m_dislocation_separation);
		sim.SetDt(dialog.m_dt);
		sim.SetKx(dialog.m_kx);
		sim.SetKy(dialog.m_ky);
		sim.SetNoise(dialog.m_noise);
		sim.SetSeed(dialog.m_seed);
		sim.SetMin(dialog.m_min);
		sim.SetMax(dialog.m_max);

	// Swift-Hohenberg
		sim.SetEpsilon(dialog.m_epsilon);
		sim.SetK(dialog.m_k);

	// Generalized Swift-Hohenberg
		sim.SetG_m(dialog.m_g_m);
		sim.SetC2(dialog.m_c2);

		sim.Restart();
		thread = AfxBeginThread(RunThread,this);
	}
}

