/* (C) Copyright 1993 by Steven Trainoff.  Permission is granted to make
* any use of this code subject to the condition that all copies contain
* this notice and an indication of what has been changed.
*/
#include <stdlib.h>
#include <stdio.h>
#include "spt.h"

extern char *progname;

/* This routine is used to shuffle an image array into "FFT" order.
 * It moves blocks of the image around so that the FFT origin is in
 * the center.  The dimensions must be multiples of 2.
 *
 * +---+---+         +---+---+
 * | 1 | 2 |         | 4 | 3 |
 * +---+---+	==>  +---+---+
 * | 3 | 4 |         | 2 | 1 |
 * +---+---+         +---+---+
 */
 
void shuffle_array(unsigned char **array, int dim[2])
{
    register int i, j;
    int otheri, otherj;

    if ((dim[0] % 2) || (dim[1] % 2)) {
	fprintf(stderr, "%s: Dimension not a multiple of 2 (%d, %d)\n", 
	    progname, dim[0], dim[1]);
	exit(-1);
    }
    /* Swap quadrants 1 and 4 */
    for (i = 0, otheri=dim[0]/2; otheri < dim[0]; i++, otheri++) {
	for (j = 0, otherj=dim[1]/2; otherj < dim[1]; j++, otherj++) {
	    unsigned char tmp = array[i][j];
	    array[i][j] = array[otheri][otherj];
	    array[otheri][otherj] = tmp;
	}
    }
    /* Swap quadrants 2 and 3 */
    for (i = 0, otheri=dim[0]/2; otheri < dim[0]; i++, otheri++) {
	for (j = dim[1]/2, otherj=0; j < dim[1]; j++, otherj++) {
	    unsigned char tmp = array[i][j];
	    array[i][j] = array[otheri][otherj];
	    array[otheri][otherj] = tmp;
	}
    }
}

/* This routine is used to transpose square arrays */
void transpose_array(unsigned char **array, int dim[2])
{
    register int i, j;
    
    if (dim[0] != dim[1]) {
    	fprintf(stderr, "%s: Array is not square (%d, %d)\n", 
	    progname, dim[0], dim[1]);
	exit(-1);
    }
    
    for (i = 0; i < dim[0]; i++) {
    	for (j = i+1; j < dim[0]; j++) {
	    unsigned char tmp = array[i][j];
	    array[i][j] = array[j][i];
	    array[j][i] = tmp;
	}
    }
}

    
